@@ -14,11 +14,13 @@ from . import tables, datasets
 
 try:
     import numpy
+    _sum = numpy.sum
 except ImportError:
     numpy = None
+    _sum = sum
 
 __all__ = ["Peptide", "tables", "datasets"]
-__version__ = "0.2.1"
+__version__ = "0.3.0"
 __author__ = "Martin Larralde <martin.larralde@embl.de>"
 __license__ = "GPLv3"
 __credits__ = """
@@ -399,6 +401,66 @@ class Peptide(typing.Sequence[str]):
             s = numpy.sum(v1*v2)
         return s / len(self)
 
+    def profile(
+        self,
+        table: typing.Dict[str, float],
+        window: int = 1,
+        default: float = 0.0,
+    ) -> typing.Sequence[float]:
+        """Compute a generic per-residue profile from per-residue indices.
+
+        Arguments:
+            table (`dict`): The values per residue to apply to the whole
+                protein sequence.
+            window (`int`): The window size for computing the profile.
+                Leave as *1* to return per-residue values.
+            default (`float`): The default value to use for amino-acids
+                that are not present in the given table.
+
+        Returns:
+            `collections.abc.Sequence` of `float`: The per-residue profile
+            values, averaged in the given window size. When ``window`` is
+            larger than the available number of resiudes, an empty sequence
+            is returned.
+
+        Example:
+            >>> peptide = Peptide("PKLVCLKKC")
+            >>> peptide.profile(peptides.tables.CHARGE['sign'])
+            [0.0, 1.0, 0.0, 0.0, -1.0, 0.0, 1.0, 1.0, -1.0]
+            >>> peptide.profile(peptides.tables.MOLECULAR_WEIGHT['expasy'], 5)
+            [108..., 111..., 111..., 114..., 115...]
+
+        .. versionadded:: 0.3.0
+
+        """
+        if window < 1:
+            raise ValueError("Window must be strictly positive")
+        # skip computing profile is window is larger than the available
+        # number of residues in the peptide sequence
+        if len(self) + 1 > window:
+            # build a look-up table and index values
+            lut = [table.get(aa, default) for aa in self._CODE1]
+            if numpy is None:
+                values = [lut[i] for i in self.encoded]
+            else:
+                values = numpy.take(lut, self.encoded)  # type: ignore
+            # don't perform window averaging if window is 1
+            if window == 1:
+                p = list(values)
+            elif window > 1:
+                p = []
+                # use a rolling sum over the window
+                s = 0.0
+                for i in range(window):
+                    s += values[i]
+                for j in range(window, len(self)):
+                    p.append(s / window)
+                    s -= values[j-window]
+                    s += values[j]
+                p.append(s / window)
+
+        return p
+
     # --- Sequence properties -----------------------------------------------
 
     def counts(self) -> typing.Dict[str, int]:
@@ -468,6 +530,10 @@ class Peptide(typing.Sequence[str]):
               :pmid:`7462208`.
 
         """
+        # NOTE(@althonos): Counting 5 times is faster than using a loop-based
+        #                  counter or a `collections.Counter` as it will use
+        #                  the C-level `memchr` if available, which is much
+        #                  faster than iterating in Python space.
         # count aliphatic residues
         ala = self.sequence.count("A") / len(self.sequence)
         val = self.sequence.count("V") / len(self.sequence)
@@ -508,8 +574,7 @@ class Peptide(typing.Sequence[str]):
               :doi:`10.1046/j.1365-2796.2003.01228.x`. :pmid:`12930229`.
 
         """
-        lut = [tables.BOMAN["Boman"].get(aa, 0.0) for aa in self._CODE1]
-        return -_takesum(lut, self.encoded) / len(self)
+        return -_sum(self.profile(tables.BOMAN["Boman"])) / len(self)
 
     def charge(self, pH: float = 7, pKscale: str = "Lehninger") -> float:
         """Compute the theoretical net charge of a peptide sequence.
@@ -948,9 +1013,7 @@ class Peptide(typing.Sequence[str]):
         table = tables.HYDROPHOBICITY.get(scale)
         if table is None:
             raise ValueError(f"Invalid hydrophobicity scale: {scale!r}")
-        lut = [table.get(aa, 0.0) for aa in self._CODE1]
-        # average the weight of each amino acid
-        return _takesum(lut, self.encoded) / len(self)
+        return _sum(self.profile(table)) / len(self)
 
     def instability_index(self) -> float:
         """Compute the instability index of a protein sequence.
@@ -1102,10 +1165,8 @@ class Peptide(typing.Sequence[str]):
                 f"Expected str or dict, found {aa_shift.__class__.__name__}"
             )
 
-        # build the lookup table
-        lut = [scale.get(aa, 0.0) for aa in self._CODE1]
         # sum the mass-shift of each amino acid
-        shift = _takesum(lut, self.encoded)
+        shift = _sum(self.profile(scale))
         # return the shift with C-terminal and N-terminal ends
         return shift + scale.get("nTer", 0.0) + scale.get("cTer", 0.0)
 
@@ -1153,12 +1214,8 @@ class Peptide(typing.Sequence[str]):
         if scale is None:
             raise ValueError(f"Invalid average weight scale: {average!r}")
 
-        # build a look-up table
-        lut = [scale.get(aa, 0.0) for aa in self._CODE1]
-        # sum the weight of each amino acid
-        mass = _takesum(lut, self.encoded)
-        # add weight of water molecules
-        mass += scale["H2O"]
+        # sum the weight of each amino acid and add weight of water molecules
+        mass = _sum(self.profile(scale)) + scale["H2O"]
         # add mass shift for labeled proteins
         if aa_shift is not None:
             mass += self.mass_shift(
@@ -1332,7 +1389,7 @@ class Peptide(typing.Sequence[str]):
 
         return profile
 
-    # --- Structural class ---------------------------------------------------
+    # --- Structural properties ----------------------------------------------
 
     def structural_class(
         self,
@@ -1541,6 +1598,35 @@ class Peptide(typing.Sequence[str]):
         # find the most likely structural class based on the distance
         return min(distances, key=distances.get)
 
+    def linker_preference_profile(
+        self, window: int = 15
+    ) -> typing.Sequence[float]:
+        """Compute the linker preference profile of a protein sequence.
+
+        The linker preference profile is a measure used as a basis for the
+        DomCut method in Suyama & Ohara (2002). The resulting profile can
+        then be used to identify putative domain boundaries in the input
+        protein, either:
+
+        - Using prior knowledge of the estimated domain count :math:`D`, in
+          which case the :math:`D-1` global minimums in the sequence can
+          be used as cutting points
+        - Without prior knowledge of the domain count, using a fixed
+          threshold to estimate the number of domains and linkers. A
+          cutoff value of :math:`-0.09` was selected by the authors
+          optimizing on the specificity / selectivity tradeoff.
+
+        References:
+            - Suyama, M., O. Ohara O.
+              *DomCut: prediction of inter-domain linker regions in amino
+              acid sequences.* Bioinformatics. Mar 2003;19(5):673-4.
+              :doi:`10.1093/bioinformatics/btg031`. :pmid:`12651735`.
+
+        .. versionadded:: 0.3.0
+
+        """
+        return self.profile(tables.LINKER_INDEX["Suyama"], window=window)
+
     # --- Descriptors --------------------------------------------------------
 
     def blosum_indices(self) -> BLOSUMIndices:
@@ -1578,11 +1664,8 @@ class Peptide(typing.Sequence[str]):
         """
         out = []
         for i in range(len(tables.BLOSUM)):
-            # build a look-up table
-            scale = tables.BLOSUM[f"BLOSUM{i+1}"]
-            lut = [scale.get(aa, 0.0) for aa in self._CODE1]
-            # average the weight of each amino acid
-            out.append(_takesum(lut, self.encoded) / len(self))
+            p = self.profile(tables.BLOSUM[f"BLOSUM{i+1}"])
+            out.append(_sum(p) / len(self))
         return BLOSUMIndices(*out)
 
     def cruciani_properties(self) -> CrucianiProperties:
@@ -1618,11 +1701,8 @@ class Peptide(typing.Sequence[str]):
         """
         out = []
         for i in range(len(tables.CRUCIANI)):
-            # build a look-up table
-            scale = tables.CRUCIANI[f"PP{i+1}"]
-            lut = [scale.get(aa, 0.0) for aa in self._CODE1]
-            # average the weight of each amino acid
-            out.append(_takesum(lut, self.encoded) / len(self))
+            p = self.profile(tables.CRUCIANI[f"PP{i+1}"])
+            out.append(_sum(p) / len(self))
         return CrucianiProperties(*out)
 
     def fasgai_vectors(self) -> FasgaiVectors:
@@ -1661,11 +1741,8 @@ class Peptide(typing.Sequence[str]):
         """
         out = []
         for i in range(len(tables.FASGAI)):
-            # build a look-up table
-            scale = tables.FASGAI[f"F{i+1}"]
-            lut = [scale.get(aa, 0.0) for aa in self._CODE1]
-            # average the weight of each amino acid
-            out.append(_takesum(lut, self.encoded) / len(self))
+            p = self.profile(tables.FASGAI[f"F{i+1}"])
+            out.append(_sum(p) / len(self))
         return FasgaiVectors(*out)
 
     def kidera_factors(self) -> KideraFactors:
@@ -1709,11 +1786,8 @@ class Peptide(typing.Sequence[str]):
         """
         out = []
         for i in range(len(tables.KIDERA)):
-            # build a look-up table
-            scale = tables.KIDERA[f"KF{i+1}"]
-            lut = [scale.get(aa, 0.0) for aa in self._CODE1]
-            # average the weight of each amino acid
-            out.append(_takesum(lut, self.encoded) / len(self))
+            p = self.profile(tables.KIDERA[f"KF{i+1}"])
+            out.append(_sum(p) / len(self))
         return KideraFactors(*out)
 
     def ms_whim_scores(self) -> MSWHIMScores:
@@ -1756,11 +1830,8 @@ class Peptide(typing.Sequence[str]):
         """
         out = []
         for i in range(len(tables.MSWHIM)):
-            # build a look-up table
-            scale = tables.MSWHIM[f"MSWHIM{i+1}"]
-            lut = [scale.get(aa, 0.0) for aa in self._CODE1]
-            # average the weight of each amino acid
-            out.append(_takesum(lut, self.encoded) / len(self))
+            p = self.profile(tables.MSWHIM[f"MSWHIM{i+1}"])
+            out.append(_sum(p) / len(self))
         return MSWHIMScores(*out)
 
     def pcp_descriptors(self) -> PCPDescriptors:
@@ -1799,11 +1870,8 @@ class Peptide(typing.Sequence[str]):
         """
         out = []
         for i in range(len(tables.PCP_DESCRIPTORS)):
-            # build a look-up table
-            scale = tables.PCP_DESCRIPTORS[f"E{i+1}"]
-            lut = [scale.get(aa, 0.0) for aa in self._CODE1]
-            # average the weight of each amino acid
-            out.append(_takesum(lut, self.encoded) / len(self))
+            p = self.profile(tables.PCP_DESCRIPTORS[f"E{i+1}"])
+            out.append(_sum(p) / len(self))
         return PCPDescriptors(*out)
 
     def physical_descriptors(self) -> PhysicalDescriptors:
@@ -1847,11 +1915,8 @@ class Peptide(typing.Sequence[str]):
         """
         out = []
         for i in range(len(tables.PHYSICAL_DESCRIPTORS)):
-            # build a look-up table
-            scale = tables.PHYSICAL_DESCRIPTORS[f"PD{i+1}"]
-            lut = [scale.get(aa, 0.0) for aa in self._CODE1]
-            # average the weight of each amino acid
-            out.append(_takesum(lut, self.encoded) / len(self))
+            p = self.profile(tables.PHYSICAL_DESCRIPTORS[f"PD{i+1}"])
+            out.append(_sum(p) / len(self))
         return PhysicalDescriptors(*out)
 
     def protfp_descriptors(self) -> ProtFPDescriptors:
@@ -1897,11 +1962,8 @@ class Peptide(typing.Sequence[str]):
         """
         out = []
         for i in range(len(tables.PROTFP)):
-            # build a look-up table
-            scale = tables.PROTFP[f"ProtFP{i+1}"]
-            lut = [scale.get(aa, 0.0) for aa in self._CODE1]
-            # average the weight of each amino acid
-            out.append(_takesum(lut, self.encoded) / len(self))
+            p = self.profile(tables.PROTFP[f"ProtFP{i+1}"])
+            out.append(_sum(p) / len(self))
         return ProtFPDescriptors(*out)
 
     def sneath_vectors(self) -> SneathVectors:
@@ -1940,11 +2002,8 @@ class Peptide(typing.Sequence[str]):
         """
         out = []
         for i in range(len(tables.SNEATH)):
-            # build a look-up table
-            scale = tables.SNEATH[f"SV{i+1}"]
-            lut = [scale.get(aa, 0.0) for aa in self._CODE1]
-            # average the weight of each amino acid
-            out.append(_takesum(lut, self.encoded) / len(self))
+            p = self.profile(tables.SNEATH[f"SV{i+1}"])
+            out.append(_sum(p) / len(self))
         return SneathVectors(*out)
 
     def st_scales(self) -> STScales:
@@ -1982,11 +2041,8 @@ class Peptide(typing.Sequence[str]):
         """
         out = []
         for i in range(len(tables.ST_SCALES)):
-            # build a look-up table
-            scale = tables.ST_SCALES[f"ST{i+1}"]
-            lut = [scale.get(aa, 0.0) for aa in self._CODE1]
-            # average the weight of each amino acid
-            out.append(_takesum(lut, self.encoded) / len(self))
+            p = self.profile(tables.ST_SCALES[f"ST{i+1}"])
+            out.append(_sum(p) / len(self))
         return STScales(*out)
 
     def t_scales(self) -> TScales:
@@ -2021,11 +2077,8 @@ class Peptide(typing.Sequence[str]):
         """
         out = []
         for i in range(len(tables.T_SCALES)):
-            # build a look-up table
-            scale = tables.T_SCALES[f"T{i+1}"]
-            lut = [scale.get(aa, 0.0) for aa in self._CODE1]
-            # average the weight of each amino acid
-            out.append(_takesum(lut, self.encoded) / len(self))
+            p = self.profile(tables.T_SCALES[f"T{i+1}"])
+            out.append(_sum(p) / len(self))
         return TScales(*out)
 
     def vhse_scales(self) -> VHSEScales:
@@ -2067,11 +2120,8 @@ class Peptide(typing.Sequence[str]):
         """
         out = []
         for i in range(len(tables.VHSE)):
-            # build a look-up table
-            scale = tables.VHSE[f"VHSE{i+1}"]
-            lut = [scale.get(aa, 0.0) for aa in self._CODE1]
-            # average the weight of each amino acid
-            out.append(_takesum(lut, self.encoded) / len(self))
+            p = self.profile(tables.VHSE[f"VHSE{i+1}"])
+            out.append(_sum(p) / len(self))
         return VHSEScales(*out)
 
     def z_scales(self) -> ZScales:
@@ -2112,11 +2162,8 @@ class Peptide(typing.Sequence[str]):
         """
         out = []
         for i in range(len(tables.Z_SCALES)):
-            # build a look-up table
-            scale = tables.Z_SCALES[f"Z{i+1}"]
-            lut = [scale.get(aa, 0.0) for aa in self._CODE1]
-            # average the weight of each amino acid
-            out.append(_takesum(lut, self.encoded) / len(self))
+            p = self.profile(tables.Z_SCALES[f"Z{i+1}"])
+            out.append(_sum(p) / len(self))
         return ZScales(*out)
 
     __DESCRIPTORS = {
@@ -2133,16 +2180,3 @@ class Peptide(typing.Sequence[str]):
         "VHSE": vhse_scales,
         "Z": z_scales,
     }
-
-
-
-def _takesum(table, indices):
-    """Return the sum of values from ``table`` indexed by ``indices``.
-
-    With NumPy, equivalent to ``numpy.sum(numpy.take(table, indices))``.
-
-    """
-    if numpy is None:
-        return sum(map(table.__getitem__, indices))
-    else:
-        return numpy.sum(numpy.take(table, indices))
